--[[
    @file create_cover.lua
    @brief LaTeX封面代码生成文件
    @copyright MIT
    @author 陆巍
    @version 1.0.0
    @date 20221-02-27
--]]

-- 定义CreateCover类
CreateCover = {
  code = "",
  number = "1"
}

function CreateCover:new(o, str, number)
  o = o or {}
  setmetatable(o, self, self)
  self.__index = self
  self.code = str
  self.number = number
  return o
end

-- @brief 记录日志
-- @param monitoring_data 监控数据
-- @param file 要保存的日志文件
-- @parm method 文件写入方式，0表示覆盖写入，1表示追加写入
function Logger(monitoring_data, file, method)
  local str1 = monitoring_data .. "\n\n"
  local meth = method == 0 and "w" or "a"
  local log = io.open(file, meth)
  log:write(str1)
  log:close()
end

-- @brief 生成LaTeX封面代码
function CreateCover:Create()
  -- 单独调试lua文件时的require路径与LaTeX调用时的路径是不一样的
  --package.path = "fir/lua_tools/string.lua"
  --require("FirString")
  require("fir/lua_tools/string")
  -- 为配合后面的工作，需要把用户输入的内容中的单引号换成双引号
  -- 之所以用户输入时要用单引号，是因为双引号在LaTeX传入lua数据时另有别的意思
  local user_input = string.gsub(self.code, "'", '"')
  -- Logger(user_input, "test1.log", 0)
  -- 读取模板文件内容
  local fp = io.open("fir/create_cover/cover" .. self.number .. ".tex", "r")
  local template = fp:read("*a")
  fp:close()
  -- 读取模板内容中的参数块
  local parm_block = string.match(template, "%% parameter>>>>>.*%% <<<<<parameter")
  -- parm_block = string.gsub(parm_block, "\\newcommand{", "")
  -- 定义并初始化参数列表二维数组。这里控制了支持的参数数量。
  local parm_list = {}
  for i = 0, 100 do 
    parm_list[i] = {}
    for j = 0, 1 do
      parm_list[i][j] = ""
    end
  end
  -- 逐一读取参数块中的参数，并填入参数列表数组中。
  local parm_number = FirString.Count(parm_block, "\\newcommand")
  for i = 1, parm_number
  do
    local parm = FirString.SubString(parm_block, "\\newcommand{\\", "}", i)
    parm_list[i][0] = parm
    parm_list[i][1] = FirString.SubString(parm_block, "}{", "}", i)
    -- print(i, parm_list[i][0] .. "  " .. parm_list[i][1], "\n")
    i = i + 1
  end
  -- 读取用户输入的参数并替换参数列表数组中的对应值
  for i = 1, FirString.Count(user_input, ":")
  do
    local parm = FirString.GetKeyByOrder(user_input, i)
    local value = FirString.GetValueByOrder(user_input, i)
    --print(i, param, value)
    for i = 1, parm_number
    do
      if (parm_list[i][0] == parm)
      then
        parm_list[i][1] = value
        break
      end
    end
  end
  -- 提取模板文件中实际的模板内容
  template = string.gsub(template, ".*%% template>>>>>", "")
  template = string.gsub(template, "%% <<<<<template.*", "")
  -- 把模板文件中对应的参数替换成实际值
  for i = 1, parm_number
  do
    --print(parm_list[i][0], parm_list[i][1])
    template = string.gsub(template, "\\" .. parm_list[i][0], parm_list[i][1])
  end

  print(template)

  -- 写入文件
  local fp1 = io.open("cover_temp.tex", "w")
  fp1:write(template)
  fp1:close()
end

--local cover = CreateCover:new(nil, '"TitleOne": "编译原理", "TitleTwo": "学习笔记", "OrgOne": "湖畔机器人", "OrgTwo": "LAKESIDE ROBOTICS"', "1")
--cover:Create()