--[[
    @file string.lua
    @brief lua字符串处理函数库
    @copyright MIT
    @author 陆巍
    @version 1.0.0
    @date 2021-03-05
--]]

-- 定义字符串处理模块
FirString = {}

-- @brief 统计指定字符串在字符串中出现的次数
-- @param str 原字符串
-- @param target_str 目标字符串
-- @return 返回次数
function FirString.Count(str, target_str)
  local count = 0
  for i in string.gmatch(str, target_str) do
    count = count + 1
  end
  return count
end

-- @brief 读取指定字符串边界内的字符
-- @param str 被提取字符串
-- @param start_str 起始字符串
-- @param end_str 结束字符串
-- @param n 第几次出现
-- @return 返回查到的字符串
-- @note 返回字符串的左右空格将被清除
function FirString.SubString(str, start_str, end_str, n)
  if (n <= 0)
  then
    return "Error: Parameter n is invalid."
  end
  str = string.gsub(str, ".-" .. start_str .. "%s*", "", n)
  str = string.gsub(str, end_str .. ".*", "")
  return str
end

-- @brief 查找某个字符串第n次出现的位置
-- @param str 被提取字符串
-- @param n 顺序号
-- @return 返回查到的位置
function FirString.FindByOrder(str, target_str, n)
  local i = 0
  local site = 0
  while (i < n)
  do
    site = string.find(str, target_str, site + 1)
    i = i + 1
  end
  return site
end

-- @brief 通过序号获取类JSON数据的键
-- @param str 被提取字符串
-- @param n 顺序号
-- @return 返回查到的键
function FirString.GetKeyByOrder(str, n)
  if (n <= 0)
  then
    return "Error: Parameter n is invalid."
  end
  local site = FirString.FindByOrder(str, ":", n)
  str = string.sub(str, 1, site)
  str = string.gsub(str, '"%s*:', '')
  str = string.gsub(str, '.*"', '')
  return str
end

-- @brief 通过序号获取类JSON数据的值
-- @param str 被提取字符串
-- @param n 顺序号
-- @return 返回查到的值
function FirString.GetValueByOrder(str, n)
  if (n <= 0)
  then
    return "Error: Parameter n is invalid."
  end
  local site = FirString.FindByOrder(str, ":", n)
  str = string.sub(str, site)
  str = string.gsub(str, ':%s*"', '')
  str = string.gsub(str, '".*', '')
  return str
end

return FirString